/**************************************************************************
 * This program is Copyright (C) 1986-2002 by Jonathan Payne.  JOVE is    *
 * provided by Jonathan and Jovehacks without charge and without          *
 * warranty.  You may copy, modify, and/or distribute JOVE, provided that *
 * this notice is included in all the source files and documentation.     *
 **************************************************************************/

extern void
	skip_wht_space(void);

/* Commands: */

extern void
	ChrToOct(void),
	GoLine(void),
	KillBos(void),
	KillEOL(void),
	KillEos(void),
	KillExpr(void),
	Leave(void),
	NotModified(void),
	SetLMargin(void),
	SetRMargin(void),
	StrLength(void),
	ToIndent(void),
	TransChar(void),
	TransLines(void),
	Yank(void),
	prCTIME(void);
